/**
 * The class that implements MyPair; pairs are immutable.
 * 
 * @param <F> the type of the MyPair
 *
 * @author Gloss
 * @version 1.0 - 27/Nov/2012
 **/
public class MyPair<F>{

	/**
	 * The first pair element.
	 **/
	private F fst;
	
	/**
	 * The second pair element.
	 **/
	private F snd;

	/**
	 * The constructor.
	 * 
	 * @param fst - the first pair element.
	 * @param snd - the second pair element.
	 **/
	public MyPair(F fst, F snd){
		this.fst = fst;
		this.snd = snd;
	}

	/**
	 * Obtains the first element of the pair.
	 *  
	 * @return the first element of the pair.
	 **/
	public F getFst(){
		return fst;
	}

	/**
	 * Obtains the second element of the pair.
	 *  
	 * @return the second element of the pair.
	 **/
	public F getScnd(){
		return snd;
	}

	/**
	 * Obtains a new inverted pair.
	 * 
	 * @return the pair inverted.
	 **/
	public MyPair<F> getSym(){
		return new MyPair<F>(snd,fst);
	}

	/**
	 * The hashCode for the pair.
	 * 
	 * @return the pair hashCode.
	 **/
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((fst == null) ? 0 : fst.hashCode());
		result = prime * result + ((snd == null) ? 0 : snd.hashCode());
		return result;
	}

	/**
	 * Compare this pair with a given object.
	 * 
	 *  @param obj - the given object.
	 *  @return true if the given object is equal to this pair, otherwise return false.
	 **/
	@SuppressWarnings("unchecked")
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof MyPair<?>))
			return false;
		MyPair<F> other = (MyPair<F>) obj;
		if (fst == null) {
			if (other.fst != null)
				return false;
		} else if (!fst.equals(other.fst))
			return false;
		if (snd == null) {
			if (other.snd != null)
				return false;
		} else if (!snd.equals(other.snd))
			return false;
		return true;
	}
	
	/**
	 * The pair textual representation.
	 * 
	 * @return the String that represents the pair.
	 **/
	@Override
	public String toString() {
		return "<"+fst+","+snd+">";
	}
}
